:root {
  --docsearch-primary-color: var(--color-brand-primary);
  --docsearch-searchbox-background: var(--color-sidebar-search-background);
  --docsearch-key-gradient: none;
}


.DocSearch-Button {
  margin: 0;
  border-radius: 0;
  width: 100%;
  border: none;
  border-top: 1px solid var(--color-sidebar-search-border);
  border-bottom: 1px solid var(--color-sidebar-search-border);
}

.DocSearch-Button-Key {
  font-size: 12px !important;
  display: flex !important;
}
